import { EventApi, EventInput } from "@fullcalendar/core";
import { OFCEvent } from "./types";

import { DateTime, Duration } from "luxon";

const parseTime = (time: string): Duration | null => {
	let parsed = DateTime.fromFormat(time, "h:mm a");
	if (parsed.invalidReason) {
		parsed = DateTime.fromFormat(time, "HH:mm");
	}

	if (parsed.invalidReason) {
		console.error(
			`FC: Error parsing time string '${time}': ${parsed.invalidReason}'`
		);
		return null;
	}

	return Duration.fromISOTime(
		parsed.toISOTime({
			includeOffset: false,
			includePrefix: false,
		})
	);
};

const normalizeTimeString = (time: string): string | null => {
	const parsed = parseTime(time);
	if (!parsed) {
		return null;
	}
	return parsed.toISOTime({
		suppressMilliseconds: true,
		includePrefix: false,
		suppressSeconds: true,
	});
};

const add = (date: DateTime, time: Duration): DateTime => {
	let hours = time.hours;
	let minutes = time.minutes;
	return date.set({ hour: hours, minute: minutes });
};

const getTime = (date: Date): string =>
	DateTime.fromJSDate(date).toISOTime({
		suppressMilliseconds: true,
		includeOffset: false,
		suppressSeconds: true,
	});

const getDate = (date: Date): string => DateTime.fromJSDate(date).toISODate();

const combineDateTimeStrings = (date: string, time: string): string | null => {
	const parsedDate = DateTime.fromISO(date);
	if (parsedDate.invalidReason) {
		console.error(
			`FC: Error parsing time string '${date}': ${parsedDate.invalidReason}`
		);
		return null;
	}

	const parsedTime = parseTime(time);
	if (!parsedTime) {
		return null;
	}

	return add(parsedDate, parsedTime).toISO();
};

const DAYS = "UMTWRFS";

export function dateEndpointsToFrontmatter(
	start: Date,
	end: Date,
	allDay: boolean
): Partial<OFCEvent> {
	const date = getDate(start);
	const endDate = getDate(end);
	return {
		type: "single",
		date,
		endDate: date !== endDate ? endDate : undefined,
		allDay,
		...(allDay
			? {}
			: {
					startTime: getTime(start),
					endTime: getTime(end),
			  }),
	};
}

export function toEventInput(
	id: string,
	frontmatter: OFCEvent
): EventInput | null {
	let event: EventInput = {
		id,
		title: frontmatter.title,
		allDay: frontmatter.allDay,
	};
	if (frontmatter.type === "recurring") {
		event = {
			...event,
			daysOfWeek: frontmatter.daysOfWeek.map((c) => DAYS.indexOf(c)),
			startRecur: frontmatter.startRecur,
			endRecur: frontmatter.endRecur,
			extendedProps: { isTask: false },
		};
		if (!frontmatter.allDay) {
			event = {
				...event,
				startTime: normalizeTimeString(frontmatter.startTime || ""),
				endTime: frontmatter.endTime
					? normalizeTimeString(frontmatter.endTime)
					: undefined,
			};
		}
	} else {
		if (!frontmatter.allDay) {
			const start = combineDateTimeStrings(
				frontmatter.date,
				frontmatter.startTime
			);
			if (!start) {
				return null;
			}
			let end = undefined;
			if (frontmatter.endTime) {
				end = combineDateTimeStrings(
					frontmatter.endDate || frontmatter.date,
					frontmatter.endTime
				);
				if (!end) {
					return null;
				}
			}

			event = {
				...event,
				start,
				end,
				extendedProps: {
					isTask:
						frontmatter.completed !== undefined &&
						frontmatter.completed !== null,
					taskCompleted: frontmatter.completed,
				},
			};
		} else {
			event = {
				...event,
				start: frontmatter.date,
				end: frontmatter.endDate || undefined,
				extendedProps: {
					isTask:
						frontmatter.completed !== undefined &&
						frontmatter.completed !== null,
					taskCompleted: frontmatter.completed,
				},
			};
		}
	}

	return event;
}

export function fromEventApi(event: EventApi): OFCEvent {
	const isRecurring: boolean = event.extendedProps.daysOfWeek !== undefined;
	const startDate = getDate(event.start as Date);
	const endDate = getDate(event.end as Date);
	return {
		title: event.title,
		...(event.allDay
			? { allDay: true }
			: {
					allDay: false,
					startTime: getTime(event.start as Date),
					endTime: getTime(event.end as Date),
			  }),

		...(isRecurring
			? {
					type: "recurring",
					daysOfWeek: event.extendedProps.daysOfWeek.map(
						(i: number) => DAYS[i]
					),
					startRecur:
						event.extendedProps.startRecur &&
						getDate(event.extendedProps.startRecur),
					endRecur:
						event.extendedProps.endRecur &&
						getDate(event.extendedProps.endRecur),
			  }
			: {
					type: "single",
					date: startDate,
					...(startDate !== endDate ? { endDate } : {}),
					completed: event.extendedProps.taskCompleted,
			  }),
	};
}
